<?php

namespace Drupal\twitter_username\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\twitter_username\Plugin\Field\FieldType\TwitterUsername;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'twitter_username_textfield' widget.
 *
 * @FieldWidget(
 *   id = "twitter_username_textfield",
 *   label = @Translation("Twitter username textfield"),
 *   field_types = {
 *     "twitter_username"
 *   },
 * )
 */
class TwitterUsernameDefaultWidget extends WidgetBase {

  use MessengerTrait;
  use LoggerChannelTrait;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a new ModerationStateWidget object.
   *
   * @param string $plugin_id
   *   Plugin id.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   Field definition.
   * @param array $settings
   *   Field settings.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The Guzzle HTTP client.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, ClientInterface $http_client) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->httpClient = $http_client;
    $this->logger = $this->getLogger('twitter_username');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('http_client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['value'] = $element + [
      '#type' => 'textfield',
      '#default_value' => $items[$delta]->value ?? NULL,
      '#size' => 60,
      '#placeholder' => $this->getSetting('placeholder'),
      '#maxlength' => TwitterUsername::TWITTER_USERNAME_MAX_LENGTH,
      '#field_prefix' => "@",
      '#element_validate' => [[$this, 'validateElement']],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'validate_existance' => 0,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    $element['validate_existance'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Ensure the twitter username exists'),
      '#description' => $this->t('Use the Twitter API to ensure the username actually exists. Note that this is an expensive network call. To avoid timeouts only use this when you have a limited amount of field values. If the Twitter API is not reachable a watchdog error will be logged and the name accepted.'),
      '#default_value' => $this->getSetting('validate_existance'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $validate = $this->getSetting('validate_existance') ? 'On' : 'Off';
    $summary[] = $this->t('Validate twitter username: @validate', ['@validate' => $validate]);
    return $summary;
  }

  /**
   * Form element validate handler for Twitter username.
   */
  public function validateElement($element, FormStateInterface $form_state) {
    if ($twitter_username = $element['#value']) {
      // Ensure the username contains only valid characters.
      if (!preg_match('/^[A-Za-z0-9_]+$/', $twitter_username)) {
        $form_state->setError($element, $this->t('Invalid twitter username (alphanumerics only)'));
      }
    }

    // I'm sure this is not the way to get the field widget settings..
    $field_settings = $element['#element_validate'][0][0];

    if ($validate = $field_settings->getSetting('validate_existance')) {
      try {
        // Query the Twitter User page.
        // Since v1.1, we could not request the API with OAuth token.
        $this->httpClient->get('https://twitter.com/' . $twitter_username, ['method' => 'HEAD']);
      }
      catch (RequestException $e) {
        $response_code = $e->getCode();

        // HTTP status code 404 means the username doesn't exist.
        if ($response_code == 404) {
          $form_state->setError($element, $this->t("The twitter username doesn't exist."));
        }
        // Log and display an  error if we get an unexpected status code.
        else {
          $message = "The Twitter API returned the unexpected status code %code. That means it's not guaranteed the username %username actually exists.";
          $message_args = [
            '%code' => $response_code,
            '%username' => $twitter_username,
          ];

          $this->messenger()->addWarning($this->t("The Twitter API returned the unexpected status code %code. That means it's not guaranteed the username %username actually exists.", $message_args));
          $this->logger->notice($message, $message_args);
        }
      }
    }
  }

}
